﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment.Algorithms.Evolutionary {
  [Item("Evolution Strategy (GQAP) Context", "Context for Evolution Strategies (GQAP).")]
  [StorableType("8A811131-337D-4CF6-A938-FFA08BF50B40")]
  public sealed class ESContext : PopulationContext<ISingleObjectiveSolutionScope<ESGQAPSolution>> {
    
    [Storable]
    private IValueParameter<GQAP> problem;
    public GQAP Problem {
      get { return problem.Value; }
      set { problem.Value = value; }
    }

    [Storable]
    private IValueParameter<ESGQAPSolution> bestSolution;
    public ESGQAPSolution BestSolution {
      get { return bestSolution.Value; }
      set { bestSolution.Value = value; }
    }

    [Storable]
    private IValueParameter<IRandom> normalRand;
    public IRandom NormalRand {
      get { return normalRand.Value; }
      set { normalRand.Value = value; }
    }
    
    [StorableConstructor]
    private ESContext(StorableConstructorFlag _) : base(_) { }
    private ESContext(ESContext original, Cloner cloner)
      : base(original, cloner) {
      problem = cloner.Clone(original.problem);
      bestSolution = cloner.Clone(original.bestSolution);
      normalRand = cloner.Clone(original.normalRand);
    }
    public ESContext() : this("Evolution Strategy (GQAP) Context") { }
    public ESContext(string name) : base(name) {
      Parameters.Add(problem = new ValueParameter<GQAP>("Problem"));
      Parameters.Add(bestSolution = new ValueParameter<ESGQAPSolution>("BestFoundSolution"));
      Parameters.Add(normalRand = new ValueParameter<IRandom>("NormalRand"));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ESContext(this, cloner);
    }

    public ISingleObjectiveSolutionScope<ESGQAPSolution> ToScope(ESGQAPSolution code, double fitness = double.NaN) {
      var name = Problem.Encoding.Name;
      var scope = new SingleObjectiveSolutionScope<ESGQAPSolution>(code,
        name + "Solution", fitness, Problem.Evaluator.QualityParameter.ActualName) {
        Parent = Scope
      };
      scope.Variables.Add(new Variable(name, code.Assignment));
      scope.Variables.Add(new Variable("Evaluation", code.Evaluation));
      scope.Variables.Add(new Variable("StrategyParameter", code.sParam));
      return scope;
    }
  }
}
